import {
  ComponentDef,
  EndpointDef,
  SystemDef,
} from '../systemGenService';
import * as path from 'path';
import * as fs from 'fs';
import {createMapperFunc} from "./mapper-creator";
import {uppercaseFirstLetter, lowercaseFirstLetter} from "./helpers";
import {createServiceFunc} from "./service-creator";
import {createRoutesFunc} from "./routes-creator";

const ncp = require('ncp').ncp;


export function createViews(systemDef: SystemDef) {
  let viewsPromises = [];
  for (let i in systemDef.components) {
    viewsPromises.push(createComponent(systemDef.components[i], systemDef));
  }
  return Promise.all(viewsPromises);
}

function createComponent(component: ComponentDef, systemDef: SystemDef) {
  return new Promise<void>((componentResolve, componentReject) => {
    ncp(path.join(process.cwd(), 'frame', 'src', 'components', '{{component}}'), path.join(process.cwd(), systemDef.name, 'src', 'components', component.component), (err: any) => {
      if (err) {
        console.log(err);
      } else {
        let mapperPromise = initializeComponentFile(path.join(process.cwd(), systemDef.name, 'src', 'components', component.component, '{{component}}Mapper.ets'), path.join(process.cwd(), systemDef.name, 'src', 'components', component.component, `${component.component}Mapper.ts`), component.component, systemDef.name);
        for (let i in component.endpoints) {
          mapperPromise = mapperPromise.then(() => {
            return insertMapperCode(component.endpoints[i], systemDef.name);
          });
        }
        let routesPromise = initializeComponentFile(path.join(process.cwd(), systemDef.name, 'src', 'components', component.component, '{{component}}Routes.ets'), path.join(process.cwd(), systemDef.name, 'src', 'components', component.component, `${component.component}Routes.ts`), component.component, systemDef.name);
        for (let i in component.endpoints) {
          routesPromise = routesPromise.then(() => {
            return insertRoutesCode(component.endpoints[i], systemDef.name);
          })
        }
        addInitializeRoutesCode(component.component, systemDef.name);
        let serviceFileLocation: string = path.join(process.cwd(), systemDef.name, 'src', 'components', component.component, `${component.component}Service.ts`);
        let servicePromise = initializeComponentFile(path.join(process.cwd(), systemDef.name, 'src', 'components', component.component, '{{component}}Service.ets'), serviceFileLocation, component.component, systemDef.name);
        for (let i in component.endpoints) {
          servicePromise = servicePromise.then(() => {
            return insertServiceCode(component.endpoints[i], systemDef.name);
          });
        }
        addInitializeServiceCode(component.component, systemDef.name);
        Promise.all([mapperPromise, routesPromise, servicePromise]).then(() => {
          console.log(`success creating component ${component.component}`);
          componentResolve();
        });
      }
      // TODO: remove {{component}} folder that hasn't been used
    });
  });
}

function initializeComponentFile(sourceFile: string, destinationFile: string, component: string, outDir: string) {
  return new Promise<void>((resolve, reject) => {
    fs.rename(sourceFile, destinationFile, (err: any) => {
      let fileContents = fs.readFileSync(destinationFile, 'utf8');
      var uppercaseFirstLetterComponentName = uppercaseFirstLetter(component);
      var lowercaseFirstLetterComponentName = lowercaseFirstLetter(component);
      let newFileContents = fileContents.split('{{Component}}').join(uppercaseFirstLetterComponentName);
      newFileContents = newFileContents.split('{{component}}').join(lowercaseFirstLetterComponentName);
      fs.writeFileSync(destinationFile, newFileContents, 'utf8');
      resolve();
    });
  })
}

function addInitializeServiceCode(component: string, outDir: string) {
  let fileLocation = path.join(process.cwd(), outDir, 'src', 'initializeServices.ts');
  let initServicesFile: string = fs.readFileSync(fileLocation, 'utf8');
  let parts = initServicesFile.split('// SYSTEM-BUILDER-initialize-import');
  parts[0] = parts[0] + `import { ${component}Service } from './components/${component}/${component}Service';
`;
  let newServicesFile = parts.join('// SYSTEM-BUILDER-initialize-import');
  parts = newServicesFile.split('// SYSTEM-BUILDER-initialize-service');
  parts[0] = parts[0] + `${component}Service.initialize(),
`;
  newServicesFile = parts.join('    // SYSTEM-BUILDER-initialize-service');
  fs.writeFileSync(fileLocation, newServicesFile, 'utf8');
}

function addInitializeRoutesCode(component: string, outDir: string) {
  let fileLocation = path.join(process.cwd(), outDir, 'src', 'app.ts');
  let initRoutesFile: string = fs.readFileSync(fileLocation, 'utf8');
  let parts = initRoutesFile.split('// SYSTEM-BUILDER-router.require');
  parts[0] = parts[0] + `var ${component}Router = require('./components/${component}/${component}Routes');
`;
  let newRoutesFile = parts.join('// SYSTEM-BUILDER-router.require');
  parts = newRoutesFile.split('// SYSTEM-BUILDER-app.use');
  parts[0] = parts[0] + `app.use('/api/v1/${component}', ${component}Router);
`;
  newRoutesFile = parts.join('// SYSTEM-BUILDER-app.use');
  fs.writeFileSync(fileLocation, newRoutesFile, 'utf8');
}

function insertMapperCode(view: EndpointDef, outDir: string): Promise<void> {
  return new Promise<void>((resolve) => {
    const separator: string = `// SYSTEM-BUILDER-${view.component}-mapper`;
    let fileLocation = path.join(process.cwd(), outDir, 'src', 'components', view.component, `${view.component}Mapper.ts`);
    let initMapperFile: string = fs.readFileSync(fileLocation, 'utf8');
    let parts = initMapperFile.split(separator);
    parts[0] = parts[0] + createMapperFunc(view);
    let newMapperFile = parts.join(separator);
    fs.writeFileSync(fileLocation, newMapperFile, 'utf8');
    resolve();
  });
}

function insertServiceCode(view: EndpointDef, outDir: string): Promise<void> {
  return new Promise<void>((resolve) => {
    const separator: string = `// SYSTEM-BUILDER-${view.component}-service`;
    let fileLocation = path.join(process.cwd(), outDir, 'src', 'components', view.component, `${view.component}Service.ts`);
    let initServiceFile: string = fs.readFileSync(fileLocation, 'utf8');
    let parts = initServiceFile.split(separator);
    parts[0] = parts[0] + createServiceFunc(view);
    let newServiceFile = parts.join(separator);
    fs.writeFileSync(fileLocation, newServiceFile, 'utf8');
    resolve();
  });
}

function insertRoutesCode(view: EndpointDef, outDir: string): Promise<void> {
  return new Promise<void>((resolve) => {
    const separator: string = `// SYSTEM-BUILDER-${view.component}-routes`;
    let fileLocation = path.join(process.cwd(), outDir, 'src', 'components', view.component, `${view.component}Routes.ts`);
    let initRoutesFile: string = fs.readFileSync(fileLocation, 'utf8');
    let parts = initRoutesFile.split(separator);
    parts[0] = parts[0] + createRoutesFunc(view);
    let newRoutesFile = parts.join(separator);
    fs.writeFileSync(fileLocation, newRoutesFile, 'utf8');
    resolve();
  });
}

